@@ -139,12 +139,18 @@ class Drone:
         if self.isOpen():
             self._printLog("Closing serial port.")
 
+        self._printLog("Thread Flag False.")
+
         if self._flagThreadRun == True:
             self._flagThreadRun = False
-            sleep(0.01)
+            sleep(0.1)
+        
+        self._printLog("Thread Join.")
 
         if self._thread != None:
-            self._thread.join()
+            self._thread.join(timeout=1)
+
+        self._printLog("Port Close.")
 
         if self.isOpen() == True:
             self._serialport.close()
@@ -156,16 +162,19 @@ class Drone:
         if (header == None) or (data == None):
             return None
 
-        if (not isinstance(header, Header)) or (not isinstance(data, ISerializable)):
+        if (not isinstance(header, Header)):
             return None
 
+        if (isinstance(data, ISerializable)):
+            data = data.toArray()
+
         crc16 = CRC16.calc(header.toArray(), 0)
-        crc16 = CRC16.calc(data.toArray(), crc16)
+        crc16 = CRC16.calc(data, crc16)
 
         dataArray = bytearray()
         dataArray.extend((0x0A, 0x55))
         dataArray.extend(header.toArray())
-        dataArray.extend(data.toArray())
+        dataArray.extend(data)
         dataArray.extend(pack('H', crc16))
 
         return dataArray
